#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <stdio.h>
#include <string.h>

char *name[] = {
	"XLibex",
	NULL,
};
char str[] = "Click on me.";

int main(int argc, char **argv)
{
	Display 	*display;
	Window	  	window, button;
	int		screen;
	unsigned long 	black, white;
	XSizeHints	*sizehints;
	XWMHints	*wmhints;
	XTextProperty	wname;
	XGCValues	values;
	GC	     	gc, inv_gc;
	XEvent 		event;
	KeySym 		key;
	int    		finished = 0, inverted = 0;
	short		width = 100, height = 40;
	
/* oteven displeje */
	display = XOpenDisplay("");
	screen = DefaultScreen(display);
	black = BlackPixel(display, screen);
	white = WhitePixel(display, screen);

/* preferovana pozice a velikost */
	sizehints = XAllocSizeHints();
	sizehints->x = 100;
	sizehints->y = 100;
	sizehints->width = width;
	sizehints->min_width = width;
	sizehints->height = height;
	sizehints->min_height = height;
	sizehints->flags = PPosition | PSize | PMinSize;

/* vytvoen hlavnho okna */
	window = XCreateSimpleWindow(
		display,			/* displej */
		DefaultRootWindow(display),	/* pedek */
		sizehints->x, sizehints->y,	/* pozice */
		sizehints->width, 		/* vka okna */
		sizehints->height,		/* ka okna */
		5,				/* tlouka rmeku */
		black,				/* barva rmeku */
		white);				/* barva pozad */

/* definice vlastnost hlavnho okna pro sprvce oken */
	wmhints = XAllocWMHints();
	wmhints->input = True;
	wmhints->flags = InputHint;
	XStringListToTextProperty(name, 1, &wname);
	XSetWMProperties(
		display,			/* displej */
		window,				/* okno */
		&wname,				/* jmno okna */
 		NULL, 				/* jmno ikony */
		argv, argc,			/* parametry */
		sizehints,			/* velikost a pozice */
		wmhints,			/* vlastnosti */
		NULL);				/* tda aplikace */

/* vytvoen okna tlatka */
	button = XCreateSimpleWindow(
		display,			/* displej */
		window,				/* nadazen okno */
		10, 10,				/* pozice */
		80, 20,				/* velikost */
		1,				/* tlouka rmeku */
		black,				/* barva rmeku */
		white);				/* barva pozad */

/* vbr udlost pro okno tlatka */
	XSelectInput(display, button,
		ButtonPressMask | ButtonReleaseMask | 
		KeyPressMask | ExposureMask);
	XSelectInput(display, window, StructureNotifyMask);

/* vytvoen grafickch kontext */
	values.foreground = black;
	values.background = white;
	gc = XCreateGC(display, button,
			GCForeground|GCBackground, &values);

	values.foreground = white;
	values.background = black;
	inv_gc = XCreateGC(display, button,
			GCForeground|GCBackground, &values);

/* zobrazen okna na vrcholu */
	XMapRaised(display, window);
	XMapRaised(display, button);


/* hlavn cyklus */
	while (finished == 0) {
		XNextEvent(display, &event);
	  	switch (event.type) {
/* pri zmene velikosti vycentrovani tlacitka */
		case ConfigureNotify:
			if (event.xconfigure.width != width ||
			    event.xconfigure.height != height) {
				width = event.xconfigure.width;
				height = event.xconfigure.height;
				XMoveWindow(display, button, 
					(width>80?(width-80)/2:0),
					(height>20?(height-20)/2:0));
				printf("new width=%d, height=%d\n", width, height);
			}
			break;
		case Expose:
/* vykreslen obsahu oken */
			if (event.xexpose.count == 0) {
redraw:
				if (inverted) {
					XFillRectangle(display, button,
 						gc, 0, 0, 80, 20);
					XDrawImageString(display, button, 
						inv_gc, 5, 15,str, strlen(str));
				} else {
					XDrawImageString(display, button, 
						gc, 5, 15, str, strlen(str));
				}
			}
			break;
		case ButtonPress:
/* zmknut tlatka myi */
			if (event.xbutton.button == Button1){
				inverted = 1;
				printf("Button1 press\n");
				goto redraw;
			}
			break;

		case ButtonRelease:
			if (event.xbutton.button == Button1){
				inverted = 0;
				XClearArea(display, button, 0,0, 0, 0, True);
				printf("Button1 release\n");
			}
			if (event.xbutton.button == Button3){
				finished = 1;
			}
			break;
/* klvesa na klvesnici */
		case KeyPress:
			key = XLookupKeysym(&event.xkey, 0);
			if (key == XK_Return ||
			    key == XK_space) finished = 1;
			break;

/* zmna rozloen klvesnice */
		case MappingNotify:
			XRefreshKeyboardMapping( (XMappingEvent *)&event);
			break;

		} /* switch */
	} 	/* while */
/* ukonceni */
	XFreeGC(display, gc);
	XFreeGC(display, inv_gc);
	XDestroyWindow(display, button);
	XDestroyWindow(display, window);
	XCloseDisplay(display);
	return 0;
}

